/* 
   The samba-bugs bug tracking system
   utility functions

   Copyright (C) Andrew Tridgell 1997

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "includes.h"

#define MAX_DIR 10000
#define MAX_LINES 10000

/* lowercase a string */
void strlower(char *str)
{
	while (*str) {
		if (isupper(*str)) *str = tolower(*str);
		str++;
	}
}


/* check if a file is a directory */
int is_directory(char *dir)
{
	struct stat st;
	if (stat(dir, &st)) return 0;
	return S_ISDIR(st.st_mode);
}


#define find_fatal { chdir(start_dir); return NULL; }
/* recursively find first matching file from a specified point in a hierachy */
char * find_file(char *start_dir, char *name)
{
    char **dir_entries;
    char *ret = NULL;
    int i;
    char curr_dir[100];
    struct stat st;

    memset(curr_dir,0,sizeof(curr_dir));
    if (chdir(start_dir)!=0) return NULL;

    if (stat(name,&st)==0) {
        if ((st.st_mode & S_IFMT) != S_IFREG) find_fatal;
	if (!getcwd(curr_dir,sizeof(curr_dir)-strlen(name))) find_fatal;
	if (!(ret=malloc(strlen(name)+strlen(curr_dir)+2))) find_fatal;
	strncpy(ret,curr_dir,strlen(curr_dir)+1);
	strcat(ret,"/");
	strcat(ret,name);
	return ret;
    } else {
        dir_entries = load_dir_list(".",is_directory);

	for (i=0;(dir_entries[i] && !ret);i++) {
	    ret = find_file(dir_entries[i],name);
	    chdir(start_dir);
	}

        free_list(dir_entries);

	if (ret) {
	    chdir(start_dir);
	    return ret;
	} else {
	    find_fatal;
	}

    }
}

/* wrap a string */
char *wrap(char *s, int ncols)
{
	char *ret = s;
	int x=0;
	char *last_space=NULL;

	if (!ret) return NULL;

	while (*s) {
		switch (*s) {
		case '\n':
			x=0;
			last_space = NULL;
			break;

		case ' ':
		case '\t':
			last_space = s;
			x++;
			break;

		default:
			x++;
			break;
		}
		if (x > ncols && last_space && x-(int)(s - last_space) > ncols/4) {
			*last_space = '\n';
			x = (s - last_space);
			last_space = NULL;
		}
		s++;
	}
	return ret;
}



/* extract an email address from a from line. Note the peculian processing
   of the < and > characters because the from line has already been through
   quotedup() */
char *extract_address(char *s)
{
	char *p1, *p2;
	static char buf[100];
	if (strlen(s) > sizeof(buf)-1) return s;

	/* first try for something with <> around it */
	p1 = strstr(s,"&lt;");
	if (p1) {
		p1 += 4;
		p2 = strstr(p1,"&gt;");
		if (p2) {
			memcpy(buf, p1, p2-p1);
			buf[(int)(p2-p1)] = 0;
			return buf;
		}
	}

	/* first try for something with <> around it */
	p1 = strstr(s,"<");
	if (p1) {
		p1 += 1;
		p2 = strstr(p1,">");
		if (p2) {
			memcpy(buf, p1, p2-p1);
			buf[(int)(p2-p1)] = 0;
			return buf;
		}
	}

	/* now try for anything with an @ in it */
	p1 = strchr(s,'@');
	if (p1) {
		while (p1 > s && !isspace(*p1)) p1--;
		if (isspace(*p1)) p1++;
		p2 = p1+1;
		while (*p2 && !isspace(*p2)) p2++;
		memcpy(buf,p1, p2-p1);
		buf[(int)(p2-p1)] = 0;
		return buf;
	}
	
	/* otherwise grab the first thing */
	p1 = s;
	while (*p1 && isspace(*p1)) p1++;
	p2 = p1;
	while (*p2 && !isspace(*p2)) p2++;
	memcpy(buf,p1, p2-p1);
	buf[(int)(p2-p1)] = 0;

	return buf;
}

/* compare two names. This is used with qsort(3) on directories or usernames*/
int namecmp(const void *n1p, const void *n2p)
{
	char **n1 = (char **)n1p;
	char **n2 = (char **)n2p;
	int i1 = atoi(*n1);
	int i2 = atoi(*n2);
	if (i1 != i2) return i1-i2;

	return strcmp(*n1, *n2);
}

/* load a directory into a list of strings. Only filenames
   which match the passed function are loaded */
char **load_dir_list(char *name, int (*fn)(char *name))
{
	char **ret;
	int i=0;
	DIR *dir;
	struct direct *d;

	ret = (char **)malloc(MAX_DIR*sizeof(ret[0]));
	if (!ret) return NULL;

	dir = opendir(name);
	if (!dir) {
		free(ret);
		return NULL;
	}

	while ((d = readdir(dir))) {
		if (d->d_name[0] == '.')
			continue;
		if (fn && !fn(d->d_name)) continue;
		ret[i] = strdup(d->d_name);
		if (!ret[i]) {
			printf("strdup failed\n");
			return NULL;
		}
		i++;
		if (i == MAX_DIR-1)
			break;
	}
	ret[i] = NULL;

	closedir(dir);

	qsort(ret, i, sizeof(ret[i]), namecmp);

	return ret;
}

/* load a file into a list of strings. Only strings
   which match the passed function are loaded */
char **load_file_list(char *name,int (*fn)(char *name))
{
	char **ret;
	int i=0;
	FILE *f;
	char buf[100];

	ret = (char **)malloc(MAX_LINES*sizeof(ret[0]));
	if (!ret) return NULL; /*should have proper fatal() for util.c*/

	f = fopen(name,"r");
	if (!f) return NULL; /*ditto fatal*/

	while (fgets(buf, sizeof(buf)-1, f)) {
		if (buf[strlen(buf)-1] == '\n')
			buf[strlen(buf)-1] = 0;
		if (!*buf || !fn(buf)) continue;
		ret[i] = strdup(buf);
		if (!ret[i]) {
		  printf("strdup failed\n"); /*ditto*/
		  return NULL;
		}
		i++;
	}
	fclose(f);

	ret[i] = NULL;

	qsort(ret, i, sizeof(ret[i]), namecmp);

	return ret;
}

/* free a list loaded by load_dir_list or load_file_list, relies on last
member being NULL */
void free_list(char **list)
{
	int i;
	if (!list) return;
	for (i=0;list[i];i++)
		free(list[i]);
	free(list);
}

/* write a file with supplied mode from a list of strings. Only strings
   which match the passed function are written. Relies on last item==NULL 
   Deletes file if no entries in the list and mode is "w".
*/
int write_list(char *fname,char *mode,char **list,int (*fn)(char *name))
{
	int i;
	FILE *f;

	for (i=0;list[i];i++);

	if (!mode || !*mode || !(f=fopen(fname,mode))) return 1;

	if (strcmp(mode,"w") && i==0) {
	        remove(fname);
	}{
	        for (i=0;list[i];i++) {
		        if (!fn(list[i])) continue;
			fprintf(f,"%s\n",list[i]);
		}
        }
	fclose(f);

	return 0;
}

/* free first line containing item from null-terminated list of strings */
int delete_list_item(char **list,char *item)
{
        int i,j,ret=1;

	if (!item || !*item) return 1;

	for (i=0; list[i] && ret; i++) {
  	    if (strstr(list[i],item)) {
	        free(list[i]);
      		list[i] = list[i+1];
		for (j=i; list[j]; j++)
	            list[j] = list [j+1];
		ret = 0;
	    } 
	}

	return ret;
}

/* append an item to a null-terminated list of strings */
int add_list_item(char **list,char *item)
{
        int i;

	if (!item || !*item) return 1;

	for (i=0; list[i]; i++);
	if (!(list[i]=strdup(item))) {
	    printf("strdup failed\n");
	    return 1;
	}
	if (!(list[i+1]=malloc(sizeof(char *)))) {
	    printf("malloc failed\n");
	    return 1;
	}
	list[i+1] = NULL;

	return 0;
}

/* pull a specified header out of a piece of email. Case insensititve: RFC822*/
char *getmailheader(char *raw,char *head, int quote)
{
	char *p1, *p2, *lraw, *lhead;
	char c;

	lraw = strdup(raw); if (!lraw) return NULL;
	lhead = strdup(head); if (!lhead) return NULL;
	strlower(lraw); strlower(lhead);

	p1 = strstr(lraw, lhead);
	if (!p1) return strdup("");
        p1 = raw + (p1 - lraw);
	free(lraw); free(lhead);

	p1 += strlen(head);
	while (*p1 && ((*p1) == ' ' || (*p1) == '\t')) p1++;
	p2 = p1;
	while (*p2 && (*p2) != '\n' && (*p2) != '\r') p2++;

	if (!(*p2)) return strdup("");

	while (p2 > p1 && isspace(*p2))
		p2--;
	if (*p2 && !isspace(*p2))
		p2++;

	c = *p2;
	(*p2) = 0;
	if (quote)
		p1 = quotedup(p1);
	else
		p1 = strdup(p1);
	(*p2) = c;
	
	return p1;
}


/* return s string showing the current date and time */
char *timestring(void)
{
	struct tm *tm;
	static char tbuf[100];
	time_t t = time(NULL);

	tm = localtime(&t);

	strftime(tbuf,sizeof(tbuf)-1,"%c", tm);
	return tbuf;
}


/* load a file as a string */
char *load_file(char *fname, struct stat *st)
{
	int fd;
	struct stat st1;
	char *ret;

	if (!st) st = &st1;

	fd = open(fname,O_RDONLY);
	if (fd == -1) return NULL;

	if (fstat(fd, st)) {
		close(fd);
		return NULL;
	}

	ret = (char *)malloc(st->st_size + 1);
	if (!ret) {
		close(fd);
		return NULL;
	}

	if (read(fd, ret, st->st_size) != st->st_size) {
		close(fd);
		free(ret);
		return NULL;
	}

	ret[st->st_size] = 0;
	close(fd);
	return ret;
}


/* save a file from a string */
void save_file(char *fname, char *s)
{
	int fd;

	fd = open(fname,O_WRONLY|O_CREAT|O_TRUNC,0644);
	if (fd == -1) return;

	write(fd, s, strlen(s));
	close(fd);
}

/* show a selection list, including a blank element */
void select_list(char **list, char *vname, char *def)
{
	int i;

	printf("<select name=%s>\n", vname);
        printf("<option %s value=\"\">\n", (def && *def)?"":"SELECTED");
	for (i=0;list && list[i];i++) {
		char *s = quotequotes(list[i]);
		printf("<option %s value=\"%s\">%s\n", 
		       (def && strcmp(def, list[i])==0) ? "SELECTED":"",
		       s, s);
		free(s);
	}
	printf("</select>\n");
}






